//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

using MindFusion.Diagramming;
using MindFusion.Diagramming.Layout;


namespace MindFusion.Diagramming.WinForms.Samples.CS.SequenceDiagram
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			diagram1.AllowSelfLoops = false;
			diagram1.AllowLinksRepeat = false;
			GlassEffect glass = new GlassEffect();
			glass.Type = GlassEffectType.Type2;
			diagram1.NodeEffects.Add(glass);
			
			Shape shape1 = new Shape(
				new ElementTemplate[]
                {
                new LineTemplate(0, 100, 80, 100),
                new LineTemplate(80, 100, 80, 90),
                new LineTemplate(80, 90, 90, 90),
                new LineTemplate(90, 90, 90, 80),
                new LineTemplate(90, 80, 100, 80),
                new LineTemplate(100, 80, 100, 0),
                new LineTemplate(100, 0, 20, 0),
                new LineTemplate(20, 0, 20, 10),
                new LineTemplate(20, 10, 10, 10),
                new LineTemplate(10, 10, 10, 20),
                new LineTemplate(10, 20, 0, 20),
                new LineTemplate(0, 20, 0, 100)
                },
				new ElementTemplate[]
                {
                new LineTemplate(10, 20, 80, 20),
                new LineTemplate(80, 20, 80, 90),
                new LineTemplate(20, 10, 90, 10),
                new LineTemplate(90, 10, 90, 80)
                },
				new ElementTemplate[]
                {
                new LineTemplate(0, 100, 80, 100),
                new LineTemplate(80, 100, 80, 20),
                new LineTemplate(80, 20, 0, 20),
                new LineTemplate(0, 20, 0, 100)
                },
				FillMode.Winding, "SequenceObject");

			Shape shape2 = new Shape(
	            new ElementTemplate[]
                {
                new LineTemplate(0, 0, 100, 0),
                new LineTemplate(100, 0, 100, 100),
                new LineTemplate(100, 100, 0, 100),
                new LineTemplate(0, 100, 0, 0)
                },
	            new ElementTemplate[]
                {
                new LineTemplate(15, 0, 15, 100),
                new LineTemplate(85, 0, 85, 100)
                },
	            new ElementTemplate[]
                {
                new LineTemplate(15, 0, 85, 0),
                new LineTemplate(85, 0, 85, 100),
                new LineTemplate(85, 100, 15, 100),
                new LineTemplate(15, 100, 15, 0)
                },
	            FillMode.Winding, "SequenceMessage");

			shape1.DisplayName = "Object";
			shape2.DisplayName = "Message";
			shapeListBox1.Shapes = new Shape[] { shape1, shape2 };
			for (int i = 0; i < 2; i++)
			{
				ShapeNode node = new ShapeNode(diagram1);
				node.HandlesStyle = HandlesStyle.DashFrame;
				node.Text = "Enter Text";
				node.AllowOutgoingLinks = false;
				node.AllowIncomingLinks = false;
				node.Shape = shape1;
				node.EnabledHandles = AdjustmentHandles.None;
                node.Brush = new MindFusion.Drawing.LinearGradientBrush(Color.White, Color.FromArgb(102, 154, 204), 90);
				NodeColection.Add(node);
				diagram1.Items.Add(node);
			}
			for (int i = 0; i < NodeColection.Count; i++)
			{
				NodeColection[i].Bounds = new RectangleF(44 * i + 10, 10, 24, 24);
			}
		}

		private void diagram1_NodeCreated(object sender, NodeEventArgs e)
		{			
			ShapeNode node = e.Node as ShapeNode;
			node.HandlesStyle = HandlesStyle.DashFrame;
			if (node.Shape.Id == "SequenceObject")
			{
				node.AllowOutgoingLinks = false;
				node.AllowIncomingLinks = false;
				node.EnabledHandles = AdjustmentHandles.None;
                node.Brush = new MindFusion.Drawing.LinearGradientBrush(Color.White, Color.FromArgb(102, 154, 204), 90);
				NodeColection.Add(node);				
			}
			for (int i = 0; i < NodeColection.Count; i++)
			{
				NodeColection[i].Bounds = new RectangleF(44 * i + 10, 10, 24, 24);
			}
			if (node.Shape.Id == "SequenceMessage")
			{
				node.Brush = new MindFusion.Drawing.LinearGradientBrush(Color.White, Color.Red,90);
				TextColection.Add(node);				
				for (int j = 0; j < NodeColection.Count; j++)
				{
					if ((node.Bounds.X > NodeColection[j].Bounds.X - 20) &&
					(node.Bounds.X < NodeColection[j].Bounds.X + 30))
					{
						node.EnabledHandles = AdjustmentHandles.ResizeBottomCenter | AdjustmentHandles.Move | AdjustmentHandles.ResizeTopCenter;
						node.Constraints.MoveDirection = DirectionConstraint.Vertical;
						node.Bounds = new RectangleF(NodeColection[j].Bounds.X + 6,
							e.Node.Bounds.Y, 12, 24);
						break;
					}
				}
			}
			diagramView1.EndEdit(true);
			diagramView1.BeginEdit(node);
			diagram1.Invalidate();
			diagramView1.Invalidate();			
		}

		private void diagram1_DrawBackground(object sender, DiagramEventArgs e)
		{
			foreach(DiagramNode d in NodeColection)
			{
				System.Drawing.Pen p = new System.Drawing.Pen(Color.Black);
				p.DashStyle = DashStyle.Dash;
				p.Width = 0.5F;
				e.Graphics.DrawLine(p, d.Bounds.X + d.Bounds.Width / 2,
					d.Bounds.Y + d.Bounds.Height,  d.Bounds.X + d.Bounds.Width / 2, 2100);
			}
		}

		private void diagram1_LinkCreated(object sender, LinkEventArgs e)
		{
			ArrangeLinks();
		}

		private void ArrangeLinks()
		{
			foreach (DiagramLink l in diagram1.Links)
			{
				if (l.Origin.Bounds.X < l.Destination.Bounds.X)
				{
					float a;
					if (l.Origin.Bounds.Y > l.Destination.Bounds.Y)
						a = l.Origin.Bounds.Y;
					else
						a = l.Destination.Bounds.Y;

					float b;
					if (l.Origin.Bounds.Y + l.Origin.Bounds.Height < l.Destination.Bounds.Y + l.Destination.Bounds.Height)
						b = l.Origin.Bounds.Y + l.Origin.Bounds.Height;
					else
						b = l.Destination.Bounds.Y + l.Destination.Bounds.Height;

					if (b >= a)
					{
						l.SegmentCount = 1;
						l.AutoRoute = false;
						l.Shape = LinkShape.Polyline;
						float y = a + ((b - a) / 2);
						l.ControlPoints[0] = new PointF(l.Origin.Bounds.X + l.Origin.Bounds.Width, y);
						l.ControlPoints[1] = new PointF(l.Destination.Bounds.X, y);
					}
					else
					{
						l.Shape = LinkShape.Cascading;
						l.ControlPoints[0] = new PointF(l.Origin.Bounds.X + l.Origin.Bounds.Width,
						l.Origin.Bounds.Y + l.Origin.Bounds.Height / 2);
						l.ControlPoints[l.ControlPoints.Count - 1] = new PointF(l.Destination.Bounds.X,
							l.Destination.Bounds.Y + l.Destination.Bounds.Height / 2);
						diagram1.RoutingOptions.StartOrientation = MindFusion.Diagramming.Orientation.Horizontal;
						diagram1.RoutingOptions.EndOrientation = MindFusion.Diagramming.Orientation.Horizontal;
						l.Route();
					}
				}
				else
				{
					float a;
					if (l.Origin.Bounds.Y > l.Destination.Bounds.Y)
						a = l.Origin.Bounds.Y;
					else
						a = l.Destination.Bounds.Y;

					float b;
					if (l.Origin.Bounds.Y + l.Origin.Bounds.Height < l.Destination.Bounds.Y + l.Destination.Bounds.Height)
						b = l.Origin.Bounds.Y + l.Origin.Bounds.Height;
					else
						b = l.Destination.Bounds.Y + l.Destination.Bounds.Height;

					if (b >= a)
					{
						l.SegmentCount = 1;
						l.AutoRoute = false;
						l.Shape = LinkShape.Polyline;
						float y = a + ((b - a) / 2);
						l.ControlPoints[0] = new PointF(l.Origin.Bounds.X, y);
						l.ControlPoints[1] = new PointF(l.Destination.Bounds.X + l.Destination.Bounds.Width, y);
					}
					else
					{
						l.Shape = LinkShape.Cascading;
						l.ControlPoints[0] = new PointF(l.Origin.Bounds.X,
						l.Origin.Bounds.Y + l.Origin.Bounds.Height / 2);
						l.ControlPoints[l.ControlPoints.Count - 1] = new PointF(l.Destination.Bounds.X + l.Destination.Bounds.Width,
							l.Destination.Bounds.Y + l.Destination.Bounds.Height / 2);
						diagram1.RoutingOptions.StartOrientation = MindFusion.Diagramming.Orientation.Horizontal;
						diagram1.RoutingOptions.EndOrientation = MindFusion.Diagramming.Orientation.Horizontal;
						l.Route();
					}
				}
				l.UpdateFromPoints();
			}
		}

		private void diagram1_NodeModified(object sender, NodeEventArgs e)
		{
			ArrangeLinks();
			ArrangeLinks();
		}

		List<DiagramNode> NodeColection = new List<DiagramNode>();
		List<DiagramNode> TextColection = new List<DiagramNode>();
	}
}